@@ -10,7 +10,7 @@
  *--------------------------------------------------------------------------*/
 
 var GvaScript = {
-  Version: '1.03'
+  Version: '1.09'
 }
 
 //----------protoExtensions.js
@@ -75,7 +75,11 @@ Object.extend(Element, {
          parent = parent.parentNode;
        }
        parent  = parent || document.body;
-       offset -= parent.offsetTop
+       
+       //offset -= parent.offsetTop
+       // commented out Jean-Christophe 18.4.07 
+       // solves a bug with autoCompleters, but new bug with choiceList ..
+       // .. TODO: need to investigate further how firefox handles offsets.
      }
 
      var min = offset - (parent.clientHeight * (100-percentage)/100);
@@ -201,11 +205,6 @@ GvaScript.fireEvent = function(/* type, elem1, elem2, ... */) {
 
 
 //----------keyMap.js
-/* Keymap handler, v0.1, Sept 2006, by laurent.dami AT justice.ge.ch.
-   See companion file "Keymap.pod" for documentation and license.
-*/
-
-// TODO : add a notion of "AntiRegex"
 
 //constructor
 GvaScript.KeyMap = function (rules) {
@@ -263,7 +262,6 @@ GvaScript.KeyMap.prototype = {
 
 
   _findInStack: function(event, stack) {
-
     for (var i = stack.length - 1; i >= 0; i--) {
       var rules = stack[i];
 
@@ -271,28 +269,33 @@ GvaScript.KeyMap.prototype = {
       var keyCode = event.keyCode>9 ? event.keyCode : ("0"+event.keyCode);
 
       var handler = rules[event.keyModifiers + event.keyName]
-                 || rules[event.keyModifiers + keyCode];
+                 || rules[event.keyModifiers + keyCode]
+                 || this._regex_handler(event, rules.REGEX, true)
+                 || this._regex_handler(event, rules.ANTIREGEX, false);
       if (handler) 
-	return handler;
+        return handler;
+    }
+    return null;
+  },
 
-      if (!rules.REGEX) 
-	continue;
+  _regex_handler: function(event, regex_rules, want_match) {
+    if (!regex_rules) return null;
+    for (var j = 0; j < regex_rules.length; j++) {
+      var rule      = regex_rules[j];
+      var modifiers = rule[0];
+      var regex     = rule[1];
+      var handler   = rule[2];
 
-      for (var j = 0; j < rules.REGEX.length; j++) {
-	var rule = rules.REGEX[j];
-        var modifiers = rule[0];
-        var regex     = rule[1];
-        var handler   = rule[2];
+      var same_modifiers = modifiers == null 
+                        || modifiers == event.keyModifiers;
 
-        // build regex if it was passed as a string
-	if (typeof(regex) == "string") 
-            regex = new RegExp("^(" + regex + ")$");
+      // build regex if it was passed as a string
+      if (typeof(regex) == "string") 
+        regex = new RegExp("^(" + regex + ")$");
 
-        var same_modifiers = modifiers == null 
-                          || modifiers == event.keyModifiers;
-	if (same_modifiers && regex.test(event.keyName))
-            return handler;
-      }
+      var match = same_modifiers && regex.test(event.keyName);
+      if ((match && want_match) || (!match && !want_match)) 
+        return handler;
     }
     return null;
   },
@@ -369,7 +372,7 @@ GvaScript.KeyMap.Prefix = function(rules) {
 
     // ... and push that handler on top of the current rules
     return function(event) {
-        this.rules.push(KeyMap.MapAllKeys(one_time_handler));
+        this.rules.push(GvaScript.KeyMap.MapAllKeys(one_time_handler));
     }
 };
 
@@ -536,6 +539,12 @@ GvaScript.TreeNavigator.prototype = {
       this.loadContent(node);
   },
 
+  toggle: function(node) {
+    if (this.isClosed(node))
+        this.open(node);
+    else
+        this.close(node);
+  },
 
   openEnclosingNodes: function (elem) {
     var node = this.enclosingNode(elem);
@@ -952,14 +961,18 @@ GvaScript.TreeNavigator.prototype = {
 
   _kpPlusHandler: function (event) {
     var selectedNode = this.selectedNode;
-    if (selectedNode && this.isClosed(selectedNode)) 
+    if (selectedNode && this.isClosed(selectedNode)) {
       this.open(selectedNode);
+      Event.stop(event);
+    }
   },
 
   _kpMinusHandler: function (event) {
     var selectedNode = this.selectedNode;
-    if (selectedNode && !this.isClosed(selectedNode)) 
+    if (selectedNode && !this.isClosed(selectedNode)) {
       this.close(selectedNode);
+      Event.stop(event);
+    }
   },
 
   _kpStarHandler: function (event) {
@@ -1051,7 +1064,8 @@ GvaScript.ChoiceList = function(choices, options) {
     classes          : {},        // see below for default classes
     idForChoices     : "CL_choice",
     keymap           : null,
-    grabfocus        : false
+    grabfocus        : false,
+    scrollCount      : 5
   };
 
 
@@ -1069,13 +1083,16 @@ GvaScript.ChoiceList = function(choices, options) {
     onmouseover : this._listOverHandler.bindAsEventListener(this),
     onclick     : this._clickHandler.bindAsEventListener(this),
     navigationRules: {
-      DOWN:   this._highlightDelta.bindAsEventListener(this, 1),
-      UP:     this._highlightDelta.bindAsEventListener(this, -1),
-      HOME:   this._highlightDelta.bindAsEventListener(this, -99999),
-      END:    this._highlightDelta.bindAsEventListener(this, 99999),
-      // TODO: PAGE_UP/DOWN : call server to get next/previous page
-      RETURN: this._returnHandler .bindAsEventListener(this),
-      ESCAPE: this._escapeHandler .bindAsEventListener(this)
+      DOWN:      this._highlightDelta.bindAsEventListener(this, 1),
+      UP:        this._highlightDelta.bindAsEventListener(this, -1),
+      PAGE_DOWN: this._highlightDelta.bindAsEventListener(this, 
+                                    this.options.scrollCount),
+      PAGE_UP:   this._highlightDelta.bindAsEventListener(this, 
+                                    -this.options.scrollCount),
+      HOME:      this._highlightDelta.bindAsEventListener(this, -99999),
+      END:       this._highlightDelta.bindAsEventListener(this, 99999),
+      RETURN:    this._returnHandler .bindAsEventListener(this),
+      ESCAPE:    this._escapeHandler .bindAsEventListener(this)
     }
   };
 };
@@ -1254,6 +1271,12 @@ GvaScript.ChoiceList.prototype = {
 /** 
 TODO: 
   - if ignorePrefix, should highlight current value (not the 1st one)
+      a) change in _updateChoicesFunction (because there might be an
+         initial value in the form)
+      b) what happens if value set programmatically ?
+      c) in _checkNewValue : do not destroy the choiceList; just update
+         the element
+
   - BUG: if strict && noBlank && Ajax server down, MSIE takes 100% CPU
   - messages : choose language
   - 'actions' are not documented because the design needs rethinking
@@ -1279,6 +1302,7 @@ GvaScript.AutoCompleter = function(datasource, options) {
     strict           : false,     // will not force to take value from choices
     blankOK          : true,
     colorIllegal     : "red",
+    scrollCount      : 5,
     actionItems      : null       // choice items to invoke javascript method
   };
 
@@ -1288,8 +1312,6 @@ GvaScript.AutoCompleter = function(datasource, options) {
     defaultOptions.caseSensitive = true;
   }
 
-
-
   this.options = Class.checkOptions(defaultOptions, options);
 
   var defaultClasses = {
@@ -1309,7 +1331,10 @@ GvaScript.AutoCompleter = function(datasource, options) {
   // focus() event; then a second set of keymap rules is pushed/popped
   // whenever the choice list is visible
   var basicHandler = this._keyPressHandler.bindAsEventListener(this);
-  var basicMap     = GvaScript.KeyMap.MapAllKeys(basicHandler);
+  var detectedKeys = /^(BACKSPACE|DELETE|.)$/;
+                   // catch any single char, plus some editing keys
+  var basicMap     = { DOWN: this._keyDownHandler.bindAsEventListener(this),
+                       REGEX: [[null, detectedKeys, basicHandler]] };
   this.keymap = new GvaScript.KeyMap(basicMap);
 
   // prepare some stuff to be reused when binding to inputElements
@@ -1369,6 +1394,13 @@ GvaScript.AutoCompleter.prototype = {
     Element.addClassName(div, this.classes.message);
   },
 
+  //
+  // TODO: TO BE REMOVED OR COMMITED TO ALIEN PACKAGE
+  //
+  setdatasource : function(datasource) {
+    this.updateChoices = this._updateChoicesFunction(datasource);
+  },
+
   fireEvent: GvaScript.fireEvent, // must be copied here for binding "this" 
 
 
@@ -1395,7 +1427,7 @@ GvaScript.AutoCompleter.prototype = {
            },
            onFailure: function(xhr) {
               autocompleter._runningAjax = null;
-              autocompleter.displayMessage("Ajax request failed");
+              autocompleter.displayMessage("pas de rponse du serveur");
            },
            onComplete: function(xhr) {
               Element.removeClassName(autocompleter.inputElement, 
@@ -1422,9 +1454,13 @@ GvaScript.AutoCompleter.prototype = {
           var regex = new RegExp("^" + this.inputElement.value,
                                  this.options.caseSensitive ? "" : "i");
           var matchPrefix = function(choice) {
-            var value = 
-              (typeof choice == "string") ? choice 
-                                          : choice[this.options.valueField];
+	    var value;
+	    switch(typeof choice) {
+	      case "object" : value = choice[this.options.valueField]; break;
+	      case "number" : value = choice.toString(10); break;
+	      case "string" : value = choice; break;
+	      default: throw new Error("unexpected type of value");
+            }
             return value.search(regex) > -1;
           };
           this.choices = datasource.select(matchPrefix.bind(this));
@@ -1455,17 +1491,56 @@ GvaScript.AutoCompleter.prototype = {
       }
     }
     if (this.options.strict) {
-      var valueOK = false;
+
+      // initially : not OK unless options.blankOK and value is empty
+      var valueOK = this.options.blankOK && this.inputElement.value == "";
+
+      // if choices are known : check if we have one of them
       if (this.choiceList) {
         var index = this.choiceList.currentHighlightedIndex;
         var legal = this._valueFromChoice(index);
-        if (legal == null && this.options.blankOK)
-          legal = "";
         valueOK = this.inputElement.value == legal;
       }
+
+      // else update choices and then check
+      else { 
+        var async = this.updateChoices(); 
+
+        // can only check if in synchronous mode
+        if (!async) {
+
+          // if got one single choice, take the canonic form of that one
+          if (this.choices.length == 1) { 
+            this.inputElement.value 
+              = this.lastValue
+              = this._valueFromChoice(0); // canonic form
+            this.fireEvent({type: "Complete", index: 0}, this.inputElement); 
+            valueOK = true;
+          }
+
+          // if got many choices and our input is "", check if it belongs there
+          else {
+            //if ( this.inputElement.value == "" && this.choices.length > 1 ) {
+            if ( this.inputElement.value && this.choices.length > 1 ) {
+              for (var i = 0; i < this.choices.length; i++) {
+                //if length of one element of choicelist is same as length of input.value
+                //then they are identical (here no need to check caseSensitive since it is
+                //being done when generating choiceList
+                if (this._valueFromChoice(i).length == this.inputElement.value.length) {
+                  this.inputElement.value 
+                      = this.lastValue
+                      = this._valueFromChoice(i); // canonic form
+                  this.fireEvent({type: "Complete", index: i}, this.inputElement); 
+                  valueOK = true;
+                  break;
+                }
+              }
+            }
+          }
+        }
+      }
+
       if (!valueOK) {
-        // this.displayMessage("not a legal value"); 
-        // this.inputElement.select(); 
         this.inputElement.style.backgroundColor = this.options.colorIllegal;
       }
     }
@@ -1474,36 +1549,34 @@ GvaScript.AutoCompleter.prototype = {
     this.inputElement = null;
   },
 
+  _keyDownHandler: function(event) { 
+    var valueLength = (this.inputElement.value || "").length; 
+    if (valueLength < this.options.minimumChars)
+      this.displayMessage("liste de choix  partir de " 
+                            + this.options.minimumChars + " caractres");
+    else 
+      this._displayChoices();
+    Event.stop(event);
+  },
 
   _keyPressHandler: function(event) { 
 
     // after a blur, we still get a keypress, so ignore it
     if (!this.inputElement) return; 
+    
+    // first give back control so that the inputElement updates itself,
+    // then come back through a timeout to update the Autocompleter
 
-    if (event.keyName == "DOWN") { // user wants the list now ==> no timeout
-      if (!this.inputElement.value || (this.inputElement.value.length < this.options.minimumChars))
-        this.displayMessage("liste de choix  partir de " 
-                            + this.options.minimumChars + " caractres");
-      else 
-        this._displayChoices();
-      Event.stop(event);
-    }
-    else if (! /^(SHIFT|CTRL|ALT|LEFT|RIGHT)$/.test(event.keyName)) {
-      // first give back control so that the inputElement updates itself,
-      // then come back through a timeout to update the Autocompleter
+    // cancel pending timeouts because we create a new one
+    if (this._timeoutId) clearTimeout(this._timeoutId);
 
-      // cancel pending timeouts because we create a new one
-      if (this._timeoutId) clearTimeout(this._timeoutId);
-
-      this._timeLastKeyPress = (new Date()).getTime(); 
-      this._timeoutId = setTimeout(this._checkNewValue.bind(this), 
-                                   this.options.autoSuggestDelay);
-      // do NOT stop the event here .. inputElement needs to get the event
-    }
+    this._timeLastKeyPress = (new Date()).getTime(); 
+    this._timeoutId = setTimeout(this._checkNewValue.bind(this), 
+                                 this.options.autoSuggestDelay);
+    // do NOT stop the event here .. inputElement needs to get the event
   },
 
 
-
   _checkNewValue: function() { 
 
     // ignore this keypress if after a blur (no input element)
@@ -1572,6 +1645,7 @@ GvaScript.AutoCompleter.prototype = {
     div.style.top       = coords[1] + dim.height + "px";
     div.style.maxHeight = this.options.maxHeight + "px";
     div.style.minWidth  = this.options.minWidth + "px";
+    div.style.zIndex    = 32767; //Seems to be the highest valide value
 
     // insert into DOM
     document.body.appendChild(div);
@@ -1607,8 +1681,16 @@ GvaScript.AutoCompleter.prototype = {
     if (this.choices.length > 0) {
       var ac = this;
       var cl = this.choiceList = new GvaScript.ChoiceList(this.choices, {
-        labelField : this.options.labelField
-        });
+        labelField : this.options.labelField,
+        scrollCount: this.options.scrollCount
+      });
+
+
+      // TODO: explain and publish method "choiceElementHTML", or redesign
+      // and make it a private method
+      if ( this.choiceElementHTML ) {
+        cl.choiceElementHTML = this.choiceElementHTML;
+      }
 
       cl.onHighlight = function(event) {
         if (ac.options.typeAhead) 
